# This file was generated by liblab | https://liblab.com/

from typing import List
from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..models.utils.cast_models import cast_models
from ..models import Value


class LayoutsService(BaseService):

    @cast_models
    def new(self, init: str = None, version: str = None) -> any:
        """Create new version (default or specified settings)

        :param init: init, defaults to None
        :type init: str, optional
        :param version: version, defaults to None
        :type version: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful Response
        :rtype: any
        """

        Validator(str).is_optional().validate(init)
        Validator(str).is_optional().validate(version)

        serialized_request = (
            Serializer(
                f"{self.base_url}/layouts/revision/new", self.get_default_headers()
            )
            .add_query("init", init, nullable=True)
            .add_query("version", version, nullable=True)
            .serialize()
            .set_method("PUT")
        )

        response = self.send_request(serialized_request)
        return response

    @cast_models
    def get(self, name: str, version: str = None) -> str:
        """Obtain the lastest layout for page with specified 'name'

        :param name: name
        :type name: str
        :param version: version, defaults to None
        :type version: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful Response
        :rtype: str
        """

        Validator(str).validate(name)
        Validator(str).is_optional().validate(version)

        serialized_request = (
            Serializer(
                f"{self.base_url}/layouts/actual/get", self.get_default_headers()
            )
            .add_query("name", name)
            .add_query("version", version, nullable=True)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return response

    @cast_models
    def set(self, name: str, value: str = None, version: str = None) -> any:
        """Remove all previous layouts for specified page with 'name' and add a new value

        :param name: name
        :type name: str
        :param value: value, defaults to None
        :type value: str, optional
        :param version: version, defaults to None
        :type version: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful Response
        :rtype: any
        """

        Validator(str).validate(name)
        Validator(str).is_optional().validate(value)
        Validator(str).is_optional().validate(version)

        serialized_request = (
            Serializer(
                f"{self.base_url}/layouts/actual/set", self.get_default_headers()
            )
            .add_query("name", name)
            .add_query("value", value, nullable=True)
            .add_query("version", version, nullable=True)
            .serialize()
            .set_method("PUT")
        )

        response = self.send_request(serialized_request)
        return response

    @cast_models
    def add(self, name: str, value: str = None, version: str = None) -> any:
        """Add a new value for specified page by 'name'

        :param name: name
        :type name: str
        :param value: value, defaults to None
        :type value: str, optional
        :param version: version, defaults to None
        :type version: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful Response
        :rtype: any
        """

        Validator(str).validate(name)
        Validator(str).is_optional().validate(value)
        Validator(str).is_optional().validate(version)

        serialized_request = (
            Serializer(
                f"{self.base_url}/layouts/actual/add", self.get_default_headers()
            )
            .add_query("name", name)
            .add_query("value", value, nullable=True)
            .add_query("version", version, nullable=True)
            .serialize()
            .set_method("PUT")
        )

        response = self.send_request(serialized_request)
        return response

    @cast_models
    def all(self, name: str = None, version: str = None) -> List[Value]:
        """Obtain a list of all layouts with specified 'name'

        :param name: name, defaults to None
        :type name: str, optional
        :param version: version, defaults to None
        :type version: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful Response
        :rtype: List[Value]
        """

        Validator(str).is_optional().validate(name)
        Validator(str).is_optional().validate(version)

        serialized_request = (
            Serializer(f"{self.base_url}/layouts/all/get", self.get_default_headers())
            .add_query("name", name, nullable=True)
            .add_query("version", version, nullable=True)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return [Value._unmap(item) for item in response]

    @cast_models
    def update(self, request_body: List[Value], version: str = None) -> any:
        """Remove previously set and add new layouts with specified 'name' fileds with layouts from 'layouts' fileds of provided list

        :param request_body: The request body.
        :type request_body: List[Value]
        :param version: version, defaults to None
        :type version: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful Response
        :rtype: any
        """

        Validator(Value).is_array().validate(request_body)
        Validator(str).is_optional().validate(version)

        serialized_request = (
            Serializer(
                f"{self.base_url}/layouts/all/update", self.get_default_headers()
            )
            .add_query("version", version, nullable=True)
            .serialize()
            .set_method("PUT")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return response

    @cast_models
    def remove(self, name: str = None, value: str = None, version: str = None) -> any:
        """Remove all layouts for specified 'name'

        :param name: name, defaults to None
        :type name: str, optional
        :param value: value, defaults to None
        :type value: str, optional
        :param version: version, defaults to None
        :type version: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful Response
        :rtype: any
        """

        Validator(str).is_optional().validate(name)
        Validator(str).is_optional().validate(value)
        Validator(str).is_optional().validate(version)

        serialized_request = (
            Serializer(
                f"{self.base_url}/layouts/all/remove", self.get_default_headers()
            )
            .add_query("name", name, nullable=True)
            .add_query("value", value, nullable=True)
            .add_query("version", version, nullable=True)
            .serialize()
            .set_method("DELETE")
        )

        response = self.send_request(serialized_request)
        return response

    @cast_models
    def display(self, name: str = None, version: str = None, format: str = None) -> any:
        """Display a list of all layouts with specified 'name'

        :param name: name, defaults to None
        :type name: str, optional
        :param version: version, defaults to None
        :type version: str, optional
        :param format: format, defaults to None
        :type format: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful Response
        :rtype: any
        """

        Validator(str).is_optional().validate(name)
        Validator(str).is_optional().validate(version)
        Validator(str).is_optional().validate(format)

        serialized_request = (
            Serializer(
                f"{self.base_url}/layouts/all/display", self.get_default_headers()
            )
            .add_query("name", name, nullable=True)
            .add_query("version", version, nullable=True)
            .add_query("format", format, nullable=True)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return response
